searchData={"items":[{"type":"module","title":"Boundary","doc":"Definition of boundaries within the application.\n\nA boundary is a named group of modules which can export some of its modules, and depend on other\nboundaries.\n\nBoundary definitions can be used in combination with `Mix.Tasks.Compile.Boundary` to restrain\ncross-module dependencies. A few examples of what you can do with boundary include:\n\n- Prevent invocations from the context layer to the web layer\n- Prevent invocations from the web layer to internal context modules\n- Prevent usage of Phoenix and Plug in the context layer\n- Limit usage of Ecto in the web layer to only Ecto.Changeset\n- Allow `:mix` modules to be used only at compile time","ref":"Boundary.html"},{"type":"module","title":"Quick example - Boundary","doc":"The following code defines boundaries for a typical Phoenix based project generated with\n`mix phx.new`.\n\n```\ndefmodule MySystem do\n  use Boundary, deps: [], exports: []\n  # ...\nend\n\ndefmodule MySystemWeb do\n  use Boundary, deps: [MySystem], exports: [Endpoint]\n  # ...\nend\n\ndefmodule MySystem.Application do\n  use Boundary, top_level?: true, deps: [MySystem, MySystemWeb]\n  # ...\nend\n```\n\nThese boundaries specify the allowed cross-boundary usage:\n\n  - Modules from `MySystemWeb` may use the `MySystem` module, but not other `MySystem.*` modules.\n  - `MySystem.Application` code may use `MySystem`, `MySystemWeb`, and `MySystemWeb.Endpoint`\n    modules.\n\nTo enforce these rules on project compilation, you need to include the compiler in `mix.exs`:\n\n```\ndefmodule MySystem.MixProject do\n  # ...\n\n  def project do\n    [\n      compilers: [:boundary] ++ Mix.compilers(),\n      # ...\n    ]\n  end\n\n  # ...\nend\n```\n\nSee `Mix.Tasks.Compile.Boundary` for more details on compilation warnings.","ref":"Boundary.html#module-quick-example"},{"type":"module","title":"Defining a boundary - Boundary","doc":"A boundary is defined via `use Boundary` expression in the root module. For example, the context\nboundary named `MySystem` can be defined as follows:\n\n```\ndefmodule MySystem do\n  use Boundary, opts\n  # ...\nend\n```","ref":"Boundary.html#module-defining-a-boundary"},{"type":"module","title":"Module classification - Boundary","doc":"Based on the existing definitions, modules are classified into boundaries. Each module can\nbelong to at most one boundary. A module doesn't need to belong to a boundary, in which case we\nsay that the module is unclassified.\n\nBoundary membership is determined from the module name. In the previous example, we defined a\nsingle boundary, called `MySystem`. This boundary will contain the root module (`MySystem`),\nas well as all modules whose name starts with `MySystem.`.\n\nIn addition, it's possible to extract some of the modules from a boundary into another boundary.\nFor example:\n\n```\ndefmodule MySystem do\n  use Boundary, opts\nend\n\ndefmodule MySystem.Endpoint do\n  use Boundary, opts\nend\n```\n\nSee the \"Nested boundaries\" section for details.\n\n#","ref":"Boundary.html#module-module-classification"},{"type":"module","title":"Mix tasks - Boundary","doc":"By convention, mix tasks have to reside in the `Mix.Tasks` namespace, which makes it harder to\nput them under the same boundary. To assist with this, boundary supports manual reclassification\nof such modules.\n\nThe advised approach is to introduce the `MySystem.Mix` boundary which can hold helper functions\nrequired by the mix tasks. With such boundary in place, you can manually classify mix tasks as:\n\n```\ndefmodule Mix.Tasks.SomeTask do\n  use Boundary, classify_to: MySystem.Mix\n  use Mix.Task\nend\n\ndefmodule Mix.Tasks.AnotherTask do\n  use Boundary, classify_to: MySystem.Mix\n  use Mix.Task\nend\n```\n\nThis way, both modules will be considered as a part of the `MySystem.Mix` boundary.\n\nNote that manual classification is allowed only for mix tasks and protocol implementations (see\nthe following section).\n\n#","ref":"Boundary.html#module-mix-tasks"},{"type":"module","title":"Protocol implementation - Boundary","doc":"Consider the following protocol implementation:\n\n```\ndefimpl String.Chars, for: MySchema, do: # ...\n```\n\nThis code will generate the module `String.Chars.MySchema`. Therefore, the module sits in a\ncompletely different \"namespace\". In addition, the desired boundary of such module can vary from\none case to another. In some cases, a protocol implementation might be a UI concern, while in\nothers, it might be a domain concern.\n\nFor these reasons, calls from a protocol implementation are by default not checked. However, you\ncan manually classify the protocol implementation, as follows:\n\n```\ndefimpl String.Chars, for: MySchema do\n  use Boundary, classify_to: MySystem\n  # ...\nend\n```\n\nIn this case, the protocol implementation is considered to be a part of the `MySystem` boundary,\nand the code will be checked for cross-boundary calls.\n\n\nNote that `:classify_to` option is only allowed for protocol implementations and mix tasks.\nOther modules can't be manually classified.","ref":"Boundary.html#module-protocol-implementation"},{"type":"module","title":"Exports - Boundary","doc":"Exports are boundary modules which can be used by modules from other boundaries. A boundary\nalways exports its root module, and it may additionally export other modules, which can be\nconfigured with the `:exports` option:\n\n```\ndefmodule MySystem do\n  use Boundary, exports: [User]\nend\n```\n\nIn this example, we're defining the `MySystem` boundary which exports the modules `MySystem`\nand `MySystem.User`. All other modules of this boundary are considered to be internal, and\nthey may not be used by modules from other boundaries.\n\n#","ref":"Boundary.html#module-exports"},{"type":"module","title":"Mass exports - Boundary","doc":"It's also possible to mass-export multiple modules with a single exports entry.\n\nFor example, let's say that we keep Ecto schemas under the `MySystem.Schemas` namespace. Now we\nwant to export all of these modules except `MySystem.Schemas.Base` which is a base module used\nby our schemas. We could list each individual schema in the exports section but that becomes\ntedious, and the `use Boundary` expression might become quite long and noisy. Instead, we can\nexport all of these modules with the `exports: [{Schemas, except: [Base]}, ...]`. This will\nexport all `MySystem.Schemas.*` modules, except for `MySystem.Schemas.Base`.\n\nYou can also export all modules of the boundary with `use Boundary, exports: :all`. To exclude\nsome modules from the export list use, `use Boundary, exports: {:all, except: [SomeMod, ...]}`.\n\nMass export is not advised in most situations. Prefer explicitly listing exported modules. If\nyour export list is long, it's a possible indication of an overly fragmented interface. Consider\ninstead consolidating the interface in the main boundary module, which would act as a facade.\nAlternatively, perhaps the boundary needs to be split.\n\nHowever, cases such as Ecto schemas present a valid exception, since these modules are typically\na part of the public context interface, since they are passed back and forth between the\ncontext and the interface layer (such as web).","ref":"Boundary.html#module-mass-exports"},{"type":"module","title":"Dependencies - Boundary","doc":"Each boundary may depend on other boundaries. These dependencies are used to define allowed\ncross-boundary module usage. A module from another boundary may only be used if:\n\n  - The callee boundary is a direct dependency of the caller boundary.\n  - The callee boundary exports the used module.\n\nFor example:\n\n```\ndefmodule MySystem do\n  use Boundary, exports: [User], deps: []\nend\n\ndefmodule MySystemWeb do\n  use Boundary, exports: [], deps: [MySystem]\nend\n```\n\nIn this example we specify the following rules:\n\n  - Code from the `MySystem` boundary can't use any module from other boundaries.\n  - Code from the `MySystemWeb` boundary may use exports of the `MySystem` boundary\n    (`MySystem` and `MySystem.User`).\n\nOf course, in-boundary cross-module dependencies are always allowed (any module may use all\nother modules from the same boundary).\n\nWhen listing deps and exports, a \"grouping\" syntax can also be used:\n\n```\nuse Boundary, deps: [Foo.{Bar, Baz}]\n```\n\n#","ref":"Boundary.html#module-dependencies"},{"type":"module","title":"External dependencies - Boundary","doc":"By default, all dependencies on modules from other OTP applications are permitted. However, you can restrain such\ndependencies by including boundaries from the external application. For example, let's say you want to limit the ecto\nusage in the web tier to only `Ecto.Changeset`. This can be specified as follows:\n\n```\ndefmodule MySystemWeb do\n  use Boundary, deps: [Ecto.Changeset]\nend\n```\n\nBoundary is able to use boundary definitions from an external application, if such exists. If an external application\ndoesn't define any boundary, you can still reference application modules. In such case, you're creating an _implicit\nboundary_. This is exactly what we're doing in the previous example. Ecto doesn't define its own boundaries, but we\ncan still include `Ecto.Changeset` in the deps list. This will create an implicit boundary of the same name which will\ninclude all of the submodules like `Ecto.Changeset.Foo`, or `Ecto.Changeset.Bar.Baz`. An implicit boundary exports all\nof its submodules. Note that you can't define implicit boundaries in applications which define their own boundaries.\n\nThe implicit boundaries are collected based on all deps of all boundaries in your application. For example, if one\nboundary specifies `Ecto.Query` as a dependency, while another references `Ecto.Query.API`, then two boundaries are\ndefined, and the latter will not be a part of the former.\n\nIn some cases you may want to completely prohibit the usage of some library. However, bear in mind that by default\ncalls to an external application are restricted only if the client boundary references at least one dep boundary from\nthat application. To force boundary to always restrict calls to some application, you can include the application in\nthe check apps list:\n\n```\ndefmodule MySystem do\n  use Boundary, check: [apps: [:plug]], deps: []\nend\n```\n\nThe check apps list contains additional applications which are always checked. Any calls to given applications must\nbe explicitly allowed via the `:deps` option.\n\nThe check list can contain an application name (atom), or a `{app_name, call_mode}` tuple, where `call_mode` is either\n`:runtime` or `:compile`. If only app name is specified, then both, runtime and compile-time calls will be checked.\n\nYou can also set a list of default apps checked for every boundary in mix.exs:\n\n```\ndefmodule MySystem.MixProject do\n  use Mix.Project\n\n  def project do\n    [\n      # ...\n      boundary: [\n        default: [\n          check: [apps: [{:mix, :runtime}]]\n        ]\n      ]\n    ]\n  end\n\n  # ...\nend\n```\n\nIn the example above, we're explicitly checking all runtime mix calls, while compile-time calls won't be checked.\n\nIn addition, you can define boundary as `:strict`:\n\n```\ndefmodule MySystem do\n  use Boundary, type: :strict\nend\n```\n\nWith this setting, boundary will report all calls to all external applications which are not explicitly allowed via the\n`:dep` option. You can also configure the strict type globally in your mix.exs:\n\n```\ndefmodule MySystem.MixProject do\n  use Mix.Project\n\n  def project do\n    [\n      # ...\n      boundary: [\n        default: [\n          type: :strict\n        ]\n      ]\n    ]\n  end\n\n  # ...\nend\n```\n\nAt this point, all boundaries will be checked with the strict mode. If you want to override this for some boundaries,\nyou can do it with `use Boundary, type: :relaxed`.\n\nNote that restraining calls to the `:elixir`, `:boundary`, and pure Erlang applications, such as\n`:crypto` or `:cowboy`, is currently not possible.\n\nIf you want to discover which external applications are used by your boundaries, you can use the helper mix task\n`Mix.Tasks.Boundary.FindExternalDeps`.\n\n#","ref":"Boundary.html#module-external-dependencies"},{"type":"module","title":"Compile-time dependencies - Boundary","doc":"By default, a dependency allows calls at both, compile time and runtime. In some cases you may want to permit calls to\nsome dependency only at compile-time. A typical example are modules from the `:mix` application. These modules are\nnot safe to be used at runtime. Limiting their usage to compile-time only can be done as follows:\n\n```\n# option 1: force check all runtime calls to mix\nuse Boundary, check: [{:mix, :runtime}]\n\n# option 2: permit `Mix` implicit boundary at compile time only\nuse Boundary, deps: [{Mix, :compile}]\n```\n\nWith such configuration, the following calls are allowed:\n\n- Function invocations at compile time (i.e. outside of any function, or in `unquote(...)`).\n- Macro invocations anywhere in the code.\n- Any invocations made from a public macro.\n\nNote that you might have some modules which will require runtime dependency on mix, such as custom mix tasks. It's\nadvised to group such modules under a common boundary, such as `MySystem.Mix`, and allow `mix` as a runtime\ndependency only in that boundary.\n\nFinally, it's worth noting that it's not possible permitting some dependency only at runtime. If a dependency is\nallowed at runtime, then it can also be used at compile time.","ref":"Boundary.html#module-compile-time-dependencies"},{"type":"module","title":"Controlling checks - Boundary","doc":"Occasionally you may need to relax the rules in some parts of the code.\n\nOne typical example is when `boundary` is introduced to the existing, possibly large project, which has many complex\ndependencies that can't be untangled trivially. In this case it may be difficult to satisfy all boundary constraints\nimmediately, and you may want to tolerate some violations.\n\nBoundary supports two mechanisms for this: dirty xrefs and ignored checks.\n\nA dirty xref is an invocation to another module that won't be checked by boundary. For example, suppose that in your\ncontext layer you invoke `MySystemWeb.Router.Helpers.some_url(...)`. If you don't have the time to clean up such\ninvocations immediately, you can add the module to the `dirty_xrefs` list:\n\n```\ndefmodule MySystem do\n  use Boundary,\n    # Invocations to these modules will not be checked.\n    dirty_xrefs: [MySystemWeb.Router.Helpers, ...]\nend\n```\n\nIn addition, you can tell boundary to avoid checking outgoing and/or incoming call for some boundary. This can be\ncontrolled with the `:check`. The default value is `check: [in: true, out: true]`, which means that all incoming and\noutgoing calls will be checked.\n\nThe `in: false` setting will allow any boundary to use modules from this boundary. The `out: false` setting will allow\nthis boundary to use any other boundary. If both options are set to false, boundary becomes ignored. These settings\ncan only be provided for top-level boundaries. If a boundary has some check disabled, it may not contain\nsub-boundaries.\n\nIgnoring checks can be useful for the test support modules. By introducing a top-level boundary for such modules (e.g.\n`MySystemTest`), and marking the in and out checks as false, you can effectively instruct boundary to avoid checking\nthe test support modules.","ref":"Boundary.html#module-controlling-checks"},{"type":"module","title":"Alias references - Boundary","doc":"Boundary can also check plain alias references (`Foo.Bar`). This check is by default disabled. To enable it, you can\ninclude `check: [aliases: true]` in global or boundary options. An alias reference is only checked if it corresponds\nto an existing module.","ref":"Boundary.html#module-alias-references"},{"type":"module","title":"Nested boundaries - Boundary","doc":"It is possible to define boundaries within boundaries. Nested boundaries allow you to further control the dependency\ngraph inside the boundary, and make some in-boundary modules private to others.\n\nLet's see this in an example. Suppose that we're building a Phoenix-powered blog engine. Our context layer,\n`BlogEngine`, exposes two modules, `Accounts` and `Articles` (note that `BlogEngine.` prefix is omitted for brevity)\nto the web tier:\n\n```\ndefmodule BlogEngine do\n  use Boundary, exports: [Accounts, Articles]\nend\n\ndefmodule BlogEngineWeb do\n  use Boundary, deps: [BlogEngine]\nend\n```\n\nBut beyond this, we want to further manage the dependencies inside the context. The context tier consists of the\nmodules `Repo`, `Articles`, `Accounts`, and `Accounts.Mailer`. We'd like to introduce the following constraints:\n\n- `Articles` can use `Accounts` (but not the other way around).\n- Both `Articles` and `Accounts` can use `Repo`, but `Repo` can't use any other module.\n- Only the `Accounts` module can use the internal `Accounts.Mailer` module.\n\nHere's how we can do that:\n\n```\ndefmodule BlogEngine.Repo do\n  use Boundary\nend\n\ndefmodule BlogEngine.Articles do\n  use Boundary, deps: [BlogEngine.{Accounts, Repo}]\nend\n\ndefmodule BlogEngine.Accounts do\n  use Boundary, deps: [BlogEngine.Repo]\nend\n```\n\nConceptually, we've built a boundary sub-tree inside `BlogEngine` which looks as:\n\n```text\nBlogEngine\n|\n+----Repo\n|\n+----Articles\n|\n+----Accounts\n```\n\nWith the following dependencies:\n\n```text\nArticles ----> Repo\n   |            ^\n   v            |\nAccounts -------+\n```\n\n#","ref":"Boundary.html#module-nested-boundaries"},{"type":"module","title":"Root module - Boundary","doc":"The root module of a sub-boundary plays a special role. This module can be exported by the parent boundary, and at the\nsame time it defines its own boundary. This can be seen in the previous example, where all three modules, `Articles`,\n`Accounts`, and `Repo` are exported by `BlogEngine`, while at the same time these modules define their own\nsub-boundaries.\n\nThis demonstrates the main purpose of sub-boundaries. They are a mechanism which allows you to control the\ndependencies within the parent boundary. The parent boundary still gets to decide which of these sub-modules it will\nexport. In this example, `Articles` and `Accounts` are exported, while `Repo` isn't. The sub-boundaries decide what\nwill they depend on themselves.\n\n#","ref":"Boundary.html#module-root-module"},{"type":"module","title":"Exporting from sub-boundaries - Boundary","doc":"The parent boundary may export modules that are exported by its sub-boundaries:\n\n```\ndefmodule BlogEngine do\n  use Boundary, exports: [Accounts, Articles, Articles.Article]\nend\n\ndefmodule BlogEngine.Articles do\n  use Boundary, deps: [BlogEngine.{Accounts, Repo}], exports: [Article]\nend\n```\n\nIn this example, `BlogEngine` exports `Articles.Article` which belongs to a sub-boundary.\n\nIf you want to export all exports of a sub-boundary, you can use the mass export syntax:\n\n```\nuse Boundary, exports: [{Articles, []}, ...]\n```\n\nThis will export the `Articles` module together with all the modules exported by the articles\nsub-boundary.\n\nThe parent boundary may not export a module that isn't exported by its owner boundary.\n\n#","ref":"Boundary.html#module-exporting-from-sub-boundaries"},{"type":"module","title":"Dependencies - Boundary","doc":"A sub-boundary inherits the deps from its ancestors by default. If you want to be more explicit, you can set the\nsub-boundary's type to `:strict`, in which case nothing is inherited by default, and sub-boundary must list its\ndeps. Ancestors deps are inherited up to the first `:strict` ancestor.\n\nWhen listing deps, a boundary may only depend on its direct siblings, its parent, and any dependency of its ancestors.\nIn other words, a boundary inherits all the constraints of its ancestors, and it can't bring in any new deps that are\nnot know to some ancestor.\n\nA boundary can't depend on its descendants. However, the modules from the parent boundary are implicitly allowed to\nuse the exports of the child sub-boundaries (but not of the descendants). This property holds even if boundary is\ndeclared as strict.\n\n##","ref":"Boundary.html#module-dependencies"},{"type":"module","title":"Cross-app dependencies - Boundary","doc":"If the external lib defines its own boundaries, you can only depend on the top-level boundaries. If implicit\nboundaries are used (app doesn't define its own boundaries), all such boundaries are considered as top-level, and you\ncan depend on any boundary from such app.\n\n#","ref":"Boundary.html#module-cross-app-dependencies"},{"type":"module","title":"Promoting boundaries to top-level - Boundary","doc":"It's possible to turn a nested boundary into a top-level boundary:\n\n```\ndefmodule BlogEngine.Application do\n  use Boundary, top_level?: true\nend\n```\n\nIn this case `BlogEngine.Application` is not considered to be a sub-boundary of `BlogEngine`. This option is\ndiscouraged because it introduces a mismatch between the namespace hierarchy, and the logical model. Conceptually,\n`BlogEngine.Application` is a sibling of `BlogEngine` and `BlogEngineWeb`, but in the namespace hierarchy it usually\nresides under the context namespace (courtesy of generators such as `mix new` and `mix phx.new`).\n\nAn alternative is to rename the module to `MySystemApp`:\n\n```\ndefmodule MySystemApp do\n  use Application\n  use Boundary, deps: [MySystem, MySystemWeb]\nend\n```\n\nThat way the namespace hierarchy will match the logical model.","ref":"Boundary.html#module-promoting-boundaries-to-top-level"},{"type":"function","title":"Boundary.all/1","doc":"Returns definitions of all boundaries of the main app.\n\nYou shouldn't access the data in this result directly, as it may change significantly without warnings. Use exported\nfunctions of this module to acquire the information you need.","ref":"Boundary.html#all/1"},{"type":"function","title":"Boundary.app/2","doc":"Returns the application of the given module.","ref":"Boundary.html#app/2"},{"type":"function","title":"Boundary.errors/2","doc":"Returns all boundary errors.","ref":"Boundary.html#errors/2"},{"type":"function","title":"Boundary.fetch/2","doc":"Returns the definition of the given boundary.","ref":"Boundary.html#fetch/2"},{"type":"function","title":"Boundary.fetch!/2","doc":"Returns the definition of the given boundary.","ref":"Boundary.html#fetch!/2"},{"type":"function","title":"Boundary.for_module/2","doc":"Returns definition of the boundary to which the given module belongs.","ref":"Boundary.html#for_module/2"},{"type":"function","title":"Boundary.get/2","doc":"Returns the definition of the given boundary.","ref":"Boundary.html#get/2"},{"type":"function","title":"Boundary.parent/2","doc":"Returns the immediate parent of the boundary, or nil if the boundary is a top-level boundary.","ref":"Boundary.html#parent/2"},{"type":"function","title":"Boundary.protocol_impl?/1","doc":"Returns true if the module is an implementation of some protocol.","ref":"Boundary.html#protocol_impl?/1"},{"type":"function","title":"Boundary.siblings?/2","doc":"Returns true if given boundaries are siblings.","ref":"Boundary.html#siblings?/2"},{"type":"function","title":"Boundary.unclassified_modules/1","doc":"Returns the collection of unclassified modules.","ref":"Boundary.html#unclassified_modules/1"},{"type":"function","title":"Boundary.view/1","doc":"","ref":"Boundary.html#view/1"},{"type":"type","title":"Boundary.dep_error/0","doc":"","ref":"Boundary.html#t:dep_error/0"},{"type":"type","title":"Boundary.error/0","doc":"","ref":"Boundary.html#t:error/0"},{"type":"type","title":"Boundary.export/0","doc":"","ref":"Boundary.html#t:export/0"},{"type":"type","title":"Boundary.mode/0","doc":"","ref":"Boundary.html#t:mode/0"},{"type":"type","title":"Boundary.name/0","doc":"","ref":"Boundary.html#t:name/0"},{"type":"type","title":"Boundary.reference_error/0","doc":"","ref":"Boundary.html#t:reference_error/0"},{"type":"type","title":"Boundary.t/0","doc":"","ref":"Boundary.html#t:t/0"},{"type":"type","title":"Boundary.view/0","doc":"","ref":"Boundary.html#t:view/0"},{"type":"task","title":"mix boundary.ex_doc_groups","doc":"Creates a `boundary.exs` holding ex_doc module group defintions.","ref":"Mix.Tasks.Boundary.ExDocGroups.html"},{"type":"task","title":"Integration with ExDoc - mix boundary.ex_doc_groups","doc":"The `boundary.exs` file can be integrated with ex_doc in your mix.exs:\n\n      def project do\n        [\n          …,\n          aliases: aliases(),\n          docs: docs()\n        ]\n      end\n\n      defp aliases do\n        [\n          …,\n          docs: [\"boundary.ex_doc_groups\", \"docs\"]\n        ]\n      end\n\n      defp docs do\n        [\n          …,\n          groups_for_modules: groups_for_modules()\n        ]\n      end\n\n      defp groups_for_modules do\n        {list, _} = Code.eval_file(\"boundary.exs\")\n        list\n      end","ref":"Mix.Tasks.Boundary.ExDocGroups.html#module-integration-with-exdoc"},{"type":"task","title":"mix boundary.find_external_deps","doc":"Prints information about external dependencies of all application boundaries.\n\nNote that `:stdlib`, `:kernel`, `:elixir`, and `:boundary` will not be included in the output.","ref":"Mix.Tasks.Boundary.FindExternalDeps.html"},{"type":"task","title":"mix boundary.spec","doc":"Prints information about all boundaries in the application.","ref":"Mix.Tasks.Boundary.Spec.html"},{"type":"task","title":"mix boundary.visualize","doc":"Generates a graphviz dot file for each non-empty boundary.","ref":"Mix.Tasks.Boundary.Visualize.html"},{"type":"task","title":"mix boundary.visualize.funs","doc":"Visualizes cross-function dependencies in a single module.\n\nUsage:\n\n    mix boundary.visualize.funs SomeModule\n\nThe graph is printed to the standard output using the [graphviz dot language](https://graphviz.org/doc/info/lang.html).","ref":"Mix.Tasks.Boundary.Visualize.Funs.html"},{"type":"task","title":"mix boundary.visualize.mods","doc":"Visualizes cross-module dependencies in one or more boundaries.\n\nUsage:\n\n    mix boundary.visualize.mods Boundary1 Boundary2 ...\n\nThe graph is printed to the standard output using the [graphviz dot language](https://graphviz.org/doc/info/lang.html).","ref":"Mix.Tasks.Boundary.Visualize.Mods.html"},{"type":"task","title":"mix compile.boundary","doc":"Verifies cross-module function calls according to defined boundaries.\n\nThis compiler reports all cross-boundary function calls which are not permitted, according to\nthe current definition of boundaries. For details on defining boundaries, see the docs for the\n`Boundary` module.","ref":"Mix.Tasks.Compile.Boundary.html"},{"type":"task","title":"Usage - mix compile.boundary","doc":"Once you have configured the boundaries, you need to include the compiler in `mix.exs`:\n\n```\ndefmodule MySystem.MixProject do\n  # ...\n\n  def project do\n    [\n      compilers: [:boundary] ++ Mix.compilers(),\n      # ...\n    ]\n  end\n\n  # ...\nend\n```\n\nWhen developing a library, it's advised to use this compiler only in `:dev` and `:test`\nenvironments:\n\n```\ndefmodule Boundary.MixProject do\n  # ...\n\n  def project do\n    [\n      compilers: extra_compilers(Mix.env()) ++ Mix.compilers(),\n      # ...\n    ]\n  end\n\n  # ...\n\n  defp extra_compilers(:prod), do: []\n  defp extra_compilers(_env), do: [:boundary]\nend\n```","ref":"Mix.Tasks.Compile.Boundary.html#module-usage"},{"type":"task","title":"Warnings - mix compile.boundary","doc":"Every invalid cross-boundary usage is reported as a compiler warning. Consider the following example:\n\n```\ndefmodule MySystem.User do\n  def auth() do\n    MySystemWeb.Endpoint.url()\n  end\nend\n```\n\nAssuming that calls from `MySystem` to `MySystemWeb` are not allowed, you'll get the following warning:\n\n```\n$ mix compile\n\nwarning: forbidden reference to MySystemWeb\n  (references from MySystem to MySystemWeb are not allowed)\n  lib/my_system/user.ex:3\n```\n\nSince the compiler emits warnings, `mix compile` will still succeed, and you can normally start\nyour system, even if some boundary rules are violated. The compiler doesn't force you to immediately\nfix these violations, which is a deliberate decision made to avoid disrupting the development flow.\n\nAt the same time, it's worth enforcing boundaries on the CI. This can easily be done by providing\nthe `--warnings-as-errors` option to `mix compile`.","ref":"Mix.Tasks.Compile.Boundary.html#module-warnings"},{"type":"function","title":"Mix.Tasks.Compile.Boundary.diagnostic/2","doc":"","ref":"Mix.Tasks.Compile.Boundary.html#diagnostic/2"},{"type":"exception","title":"Boundary.Error","doc":"","ref":"Boundary.Error.html"},{"type":"extras","title":"Boundary","doc":"# Boundary\n\n[![hex.pm](https://img.shields.io/hexpm/v/boundary.svg?style=flat-square)](https://hex.pm/packages/boundary)\n[![hexdocs.pm](https://img.shields.io/badge/docs-latest-green.svg?style=flat-square)](https://hexdocs.pm/boundary/)\n![Build Status](https://github.com/sasa1977/boundary/workflows/boundary/badge.svg)\n\nBoundary is a library which helps managing and restraining cross-module dependencies in Elixir projects. A few examples of the things you can do with boundary include:\n\n- Prevent invocations from the context layer to the web layer\n- Prevent invocations from the web layer to internal context modules\n- Prevent usage of Phoenix and Plug in the context layer\n- Limit usage of Ecto in the web layer to only Ecto.Changeset\n- Allow `:mix` modules to be used only at compile time","ref":"readme.html"},{"type":"extras","title":"Status - Boundary","doc":"This library has been used in smaller production projects for about a year, and while it has not been tested on larger projects or umbrella projects, no issues are expected in those cases. That being said, if you do run into problems, please open an issue!","ref":"readme.html#status"},{"type":"extras","title":"Documentation - Boundary","doc":"For a detailed reference see docs for [Boundary module](https://hexdocs.pm/boundary/Boundary.html) and [mix compiler](https://hexdocs.pm/boundary/Mix.Tasks.Compile.Boundary.html).","ref":"readme.html#documentation"},{"type":"extras","title":"Basic usage - Boundary","doc":"To use this library, you first need to define the boundaries of your project. A __boundary__ is a named group of one or more modules. Each boundary exports some (but not all!) of its modules, and can depend on other boundaries. During compilation, the boundary compiler will find and report all cross-module function calls which are not permitted according to the boundary configuration.\n\n#","ref":"readme.html#basic-usage"},{"type":"extras","title":"Example - Boundary","doc":"Add boundary as a dependency in mix.exs:\n\n```elixir\ndefmodule MySystem.MixProject do\n  use Mix.Project\n\n  # ...\n\n  defp deps do\n    [\n      {:boundary, \"~> 0.10\", runtime: false},\n      # ...\n    ]\n  end\n\n  # ...\nend\n```\n\nThe following code defines boundaries for a typical Phoenix based project generated with `mix phx.new`.\n\n```elixir\ndefmodule MySystem do\n  use Boundary, deps: [], exports: []\n  # ...\nend\n\ndefmodule MySystemWeb do\n  use Boundary, deps: [MySystem], exports: [Endpoint]\n  # ...\nend\n\ndefmodule MySystem.Application do\n  use Boundary, top_level?: true, deps: [MySystem, MySystemWeb]\n  # ...\nend\n```\n\nThe configuration above defines three boundaries: `MySystem`, `MySystemWeb`, and `MySystem.Application`.\n\nBoundary modules are determined automatically from the boundary name. For example, the `MySystem` boundary contains the `MySystem` module, as well as any module whose name starts with `MySystem.` (e.g. `MySystem.User`, `MySystem.User.Schema`, ...).\n\nThe configuration specifies the following rules:\n\n  - Modules residing in the `MySystemWeb` boundary are allowed to invoke functions from modules exported by the `MySystem` boundary.\n  - Modules residing in the `MySystem.Application` namespace are allowed to invoke functions from modules exported by `MySystem` and `MySystemWeb` boundaries.\n\nAll other cross-boundary calls are not permitted.\n\nNext, you need to add the mix compiler:\n\n```elixir\ndefmodule MySystem.MixProject do\n  use Mix.Project\n\n  def project do\n    [\n      compilers: [:boundary, :phoenix, :gettext] ++ Mix.compilers(),\n      # ...\n    ]\n  end\n\n  # ...\nend\n```\n\nBoundary rules are validated during compilation. For example, if we have the following code:\n\n```elixir\ndefmodule MySystem.User do\n  def auth do\n    MySystemWeb.Endpoint.url()\n  end\nend\n\n```\n\nThe compiler will emit a warning:\n\n```\n$ mix compile\n\nwarning: forbidden reference to MySystemWeb\n  (references from MySystem to MySystemWeb are not allowed)\n  lib/my_system/user.ex:3\n```\n\nThe complete working example is available [here](demos/my_system).\n\nBecause `boundary` is implemented as a mix compiler, it integrates seamlessly with editors which can work with mix compiler. For example, in VS Code with [Elixir LS](https://github.com/elixir-lsp/elixir-ls):\n\n![VS Code warning 1](images/vscode_warning_1.png)\n\n![VS Code warning 2](images/vscode_warning_2.png)\n\n\n#","ref":"readme.html#example"},{"type":"extras","title":"Restricting usage of external apps - Boundary","doc":"Boundary can also be used to manage calls to other apps, even if those apps don't define their own boundaries. For example, suppose you want to enforce the following rules:\n\n  - Only `MySystemWeb` can use phoenix modules\n  - Only `MySystem` can use ecto modules, except for Ecto.Changeset which can be used by `MySystemWeb` too\n  - Only `MySystemMix` can use mix modules at runtime. Everyone can use mix modules at compile time.\n\nBy default, boundary doesn't check calls to other apps. However, we can instruct it to check calls to the desired apps. This setting can be provided for each individual boundary, or globally. Since we want to restrict calls to these boundaries in the entire project, let's do this globally in mix.exs:\n\n```elixir\n# mix.exs\n\ndefmodule MySystem.MixProject do\n  def project do\n   [\n     boundary: [\n       default: [\n         check: [\n           apps: [:phoenix, :ecto, {:mix, :runtime}]\n         ]\n       ]\n     ],\n     # ...\n   ]\n  end\nend\n```\n\nWith these settings, boundary will check all calls to phoenix and ecto, and all runtime calls to mix. Compile-time calls to mix won't be checked.\n\nNow we need to allow the calls to these apps in our boundaries:\n\n```elixir\ndefmodule MySystemWeb do\n  use Boundary, deps: [Phoenix, Ecto.Changeset]\nend\n\ndefmodule MySystem do\n  use Boundary, deps: [Ecto, Ecto.Changeset]\nend\n\ndefmodule MySystemMix do\n  use Boundary, deps: [Mix]\nend\n```\n\nNote that in `MySystem` we're specifying both `Ecto` and `Ecto.Changeset`. This is because `Ecto.Changeset` is listed as a dep in `MySystemWeb`, and so it is treated as a separate boundary.","ref":"readme.html#restricting-usage-of-external-apps"},{"type":"extras","title":"Roadmap - Boundary","doc":"- [x] validate calls to external deps (e.g. preventing `Ecto` usage from `MySystemWeb`, or `Plug` usage from `MySystem`)\n- [x] support compile time vs runtime deps\n- [x] support nested boundaries (defining internal boundaries within a boundary)\n- [ ] support Erlang modules","ref":"readme.html#roadmap"},{"type":"extras","title":"License - Boundary","doc":"[MIT](LICENSE)","ref":"readme.html#license"}],"content_type":"text/markdown"}